#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

#include "config.h"
#include "imapfilter.h"
#include "data.h"


extern account_t *accounts;
extern filter_t *filters;

unsigned int options;		/* Program options. */
unsigned int capabilities;	/* Capabilities of mail server. */
char logfile[PATH_MAX];		/* Log file. */


/*
 * In the beginning there was main()...
 */
int main(int argc, char *argv[])
{
    int c, r;
    char *confile = NULL;	/* Configuration file. */
    account_t *ca;		/* Current account. */
    mbox_t *cm;			/* Current mailbox. */

    options = (OPTION_DETAILS_NORMAL | OPTION_NAMESPACE);
    *logfile = 0;

    while ((c = getopt(argc, argv, "c:hl:qv")) != -1) {
	switch (c) {
	case 'c':
	    confile = optarg;
	    break;
	case 'h':
	    usage();
	    exit(ERROR_UNDEFINED);
	    break;
	case 'l':
	    strncat(logfile, optarg, PATH_MAX - 1);
	    break;
	case 'q':
	    options &= OPTION_DETAILS_CLEAR;
	    options |= OPTION_DETAILS_QUIET;
	    break;
	case 'v':
	    options &= OPTION_DETAILS_CLEAR;
	    options |= OPTION_DETAILS_VERBOSE;
	    break;
	default:
	    usage();
	    exit(ERROR_UNDEFINED);
	    break;
	}
    }

    catch_signals();

    read_config(confile);
    open_logfile();

    for (ca = accounts; ca; ca = ca->next) {

#ifndef SSL_TLS
	if (init_connection(ca->server, ca->port))
#else
	if (init_connection(ca->server, ca->port, ca->ssl))
#endif
	    continue;

	r = greeting_response();

	if (r == RESPONSE_BYE || check_capabilities())
	    continue;

#ifdef DEBUG
	test();
#endif

	if (r != RESPONSE_PREAUTH && login(ca->username, ca->password))
	    continue;

	check_namespace();

	for (cm = ca->mboxes; cm; cm = cm->next)
	    if (!*cm->filters)
		mailbox_status(cm->name);
	    else if (!select_mailbox(cm->name)) {
		apply_filters(cm->filters);
		close_mailbox();
	    }
	logout();

	close_connection();
    }

    overwrite_passwords();

    close_logfile();

    exit(0);
}


/*
 * Print a very brief usage message.
 */
void usage(void)
{
    fprintf(stderr,
	    "usage: imapfilter [-hqv] [-c configfile] [-l logfile]\n");
}
